package org.kie.examples.jta;

import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceException;
import javax.transaction.UserTransaction;
import org.jboss.arquillian.junit.Arquillian;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.command.KieCommands;
import org.kie.api.io.KieResources;
import org.kie.api.persistence.jpa.KieStoreServices;
import org.kie.api.runtime.Environment;
import org.kie.api.runtime.EnvironmentName;
import org.kie.api.runtime.KieSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@RunWith(Arquillian.class)
public abstract class AbstractContainerTest {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractContainerTest.class);
    protected static final String MANAGED_CONTAINER = "managed-container";

    protected boolean persistence;
    private EntityManagerFactory emf;

    @Before
    public void createEMF() {
        try {
            emf = Persistence.createEntityManagerFactory("containerPU");
        } catch (PersistenceException ex) {
            if (ex.getMessage().equals("No Persistence provider for EntityManager named containerPU")) {
                    // https://community.jboss.org/thread/173265
                // BeforeMethod is run outside container first, therefore
                // this exception can be ignored
                LOGGER.warn("Unable to create EntityManagerFactory", ex);
            } else {
                throw ex;
            }
        }
    }

    protected Environment getEnvironment() {
        if (emf == null) {
            throw new IllegalStateException("Uninitialised EntityManagerFactory");
        }

        Environment env = getServices().newEnvironment();
        env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);

        try {
            Object transaction = new InitialContext().lookup("java:comp/UserTransaction");
            LOGGER.info("User transaction class: " + transaction.getClass());
            // transaction.getClass().getInterfaces()
            Assert.assertTrue(transaction instanceof UserTransaction);
        } catch (Exception ex) {
            LOGGER.debug("Something went wrong", ex);
            throw new RuntimeException(ex);
        }

        return env;
    }

    @After
    public void closeEMF() {
        if (emf != null) {
            emf.close();
        }
        emf = null;
    }

    protected KieSession createJPASession(KieBase kbase) {
        return getStore().newKieSession(kbase, null, getEnvironment());
    }

    protected KieSession loadJPASession(KieBase kbase, int sessionId) {
        return getStore().loadKieSession(sessionId, kbase, null, getEnvironment());
    }

    protected KieSession reloadSession(KieSession ksession) {
        int id = ksession.getId();
        KieBase kbase = ksession.getKieBase();
        ksession.dispose();

        return loadJPASession(kbase, id);
    }

    protected static KieServices getServices() {
        return KieServices.Factory.get();
    }

    protected static KieResources getResources() {
        return getServices().getResources();
    }

    protected static KieStoreServices getStore() {
        return getServices().getStoreServices();
    }

    protected static KieCommands getCommands() {
        return getServices().getCommands();
    }

    protected static KieBase getKieBase(KieBuilder kbuilder) {
        return getServices().newKieContainer(kbuilder.getKieModule().getReleaseId()).getKieBase();
    }

}
